package net.wanji.datacenter.service.impl;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.CrossDataRealtimeMapper;
import net.wanji.databus.dao.mapper.CrossLaneDataRealTimeMapper;
import net.wanji.databus.po.CrossDataRealtimePO;
import net.wanji.databus.po.CrossLaneDataRealTimePO;
import net.wanji.databus.po.LaneInfoPO;
import net.wanji.datacenter.cache.CrossLaneInfoCache;
import net.wanji.datacenter.pojo.dto.CrossPeriodDataDTO;
import net.wanji.datacenter.pojo.dto.LanePeriodicDataDTO;
import net.wanji.datacenter.service.DataProcessService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author duanruiming
 * @date 2024/11/15 19:59
 */
@Service
@Slf4j
public class DataProcessServiceImpl implements DataProcessService {

    @Resource
    private CrossLaneDataRealTimeMapper crossLaneDataRealTimeMapper;
    @Resource
    private CrossDataRealtimeMapper crossDataRealtimeMapper;

    /**
     * 车道信息处理存库
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void laneSave(String originalData) throws Exception {
        try {
            Map<String, LaneInfoPO> laneInfoMap = CrossLaneInfoCache.crossIdLaneId2Map;
            LanePeriodicDataDTO LanePeriodicDataDTO = JSONObject.parseObject(originalData, LanePeriodicDataDTO.class);
            String timeStamp = LanePeriodicDataDTO.getTimeStamp();
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
            List<LanePeriodicDataDTO.CrossLaneDataRealTimeDTO> eventList = LanePeriodicDataDTO.getEventList();
            List<CrossLaneDataRealTimePO> insertList = getInsertLaneList(laneInfoMap, timeStamp, dateFormat, eventList);
            //crossLaneDataRealTimeMapper.insertBatch(insertList);
        } catch (Exception e) {
            log.error("车道数据处理入库异常", e);
            throw new Exception(e);
        }
    }

    private static List<CrossLaneDataRealTimePO> getInsertLaneList(Map<String, LaneInfoPO> laneInfoMap, String timeStamp, SimpleDateFormat dateFormat, List<LanePeriodicDataDTO.CrossLaneDataRealTimeDTO> eventList) throws ParseException {
        List<CrossLaneDataRealTimePO> insertList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(eventList)) {
            for (LanePeriodicDataDTO.CrossLaneDataRealTimeDTO crossLaneDataRealTimeDTO : eventList) {
                String crossId = crossLaneDataRealTimeDTO.getCrossId();
                Integer dir = crossLaneDataRealTimeDTO.getDir();
                Integer laneNum = crossLaneDataRealTimeDTO.getLane_num();
                // 方向=神思dir + 1；车道号=神思laneNum + 10
                String key = crossId.concat(String.valueOf(dir + 1)).concat(String.valueOf(laneNum + 10));
                LaneInfoPO laneInfoPO = laneInfoMap.get(key);
                if (Objects.isNull(laneInfoPO)) {
                    continue;
                }
                String laneId = laneInfoPO.getId();
                crossLaneDataRealTimeDTO.setId(laneId);
                CrossLaneDataRealTimePO crossLaneDataRealTime = new CrossLaneDataRealTimePO();
                BeanUtils.copyProperties(crossLaneDataRealTimeDTO, crossLaneDataRealTime);
                Date startDate = dateFormat.parse(timeStamp);
                crossLaneDataRealTime.setStartTime(startDate);
                Integer batchTime = Integer.valueOf(String.valueOf(startDate.getTime()).substring(0, 10));
                crossLaneDataRealTime.setBatchTime(batchTime);
                insertList.add(crossLaneDataRealTime);
            }
        }
        return insertList;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void crossSave(String originalData) throws Exception {
        try {
            CrossPeriodDataDTO crossPeriodDataDTO = JSONObject.parseObject(originalData, CrossPeriodDataDTO.class);
            String timeStamp = crossPeriodDataDTO.getTimeStamp();
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
            List<CrossDataRealtimePO> eventList = crossPeriodDataDTO.getEventList();
            //crossDataRealtimeMapper.insertBatch(eventList);
        } catch (Exception e) {
            log.error("路口实时数据存库处理异常：{}", e);
            throw new RuntimeException(e);
        }
    }
}
